# coding: utf-8

"""
    Ringer Business API

    This API provides comprehensive endpoints for the Ringer Business system. It includes functionality for managing customer accounts, addresses, sites,  phone numbers, payments, users, and other business resources.  Operations typically require authentication and appropriate permissions. 

    The version of the OpenAPI document: 1.0.4
    Contact: support@ringer.tel
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictInt, StrictStr
from typing import Optional
from typing_extensions import Annotated
from ringer_business.models.available_phone_number_list_response import AvailablePhoneNumberListResponse
from ringer_business.models.paginated_phone_number_list_response import PaginatedPhoneNumberListResponse
from ringer_business.models.phone_number_create_request import PhoneNumberCreateRequest
from ringer_business.models.phone_number_update_request import PhoneNumberUpdateRequest
from ringer_business.models.schemas_phone_number_response import SchemasPhoneNumberResponse

from ringer_business.api_client import ApiClient, RequestSerialized
from ringer_business.api_response import ApiResponse
from ringer_business.rest import RESTResponseType


class SiteFeaturesPhoneNumbersApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    def assign_site_phone_number(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        phone_number_create_request: Annotated[PhoneNumberCreateRequest, Field(description="Details of the phone number to assign and its configuration.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SchemasPhoneNumberResponse:
        """Assign Phone Number to Site

        Assigns an available phone number to the specified site and configures its initial routing.

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param phone_number_create_request: Details of the phone number to assign and its configuration. (required)
        :type phone_number_create_request: PhoneNumberCreateRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._assign_site_phone_number_serialize(
            site_id=site_id,
            phone_number_create_request=phone_number_create_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "SchemasPhoneNumberResponse",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '404': "Error",
            '409': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def assign_site_phone_number_with_http_info(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        phone_number_create_request: Annotated[PhoneNumberCreateRequest, Field(description="Details of the phone number to assign and its configuration.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SchemasPhoneNumberResponse]:
        """Assign Phone Number to Site

        Assigns an available phone number to the specified site and configures its initial routing.

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param phone_number_create_request: Details of the phone number to assign and its configuration. (required)
        :type phone_number_create_request: PhoneNumberCreateRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._assign_site_phone_number_serialize(
            site_id=site_id,
            phone_number_create_request=phone_number_create_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "SchemasPhoneNumberResponse",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '404': "Error",
            '409': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def assign_site_phone_number_without_preload_content(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        phone_number_create_request: Annotated[PhoneNumberCreateRequest, Field(description="Details of the phone number to assign and its configuration.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Assign Phone Number to Site

        Assigns an available phone number to the specified site and configures its initial routing.

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param phone_number_create_request: Details of the phone number to assign and its configuration. (required)
        :type phone_number_create_request: PhoneNumberCreateRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._assign_site_phone_number_serialize(
            site_id=site_id,
            phone_number_create_request=phone_number_create_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '201': "SchemasPhoneNumberResponse",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '404': "Error",
            '409': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _assign_site_phone_number_serialize(
        self,
        site_id,
        phone_number_create_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if site_id is not None:
            _path_params['siteId'] = site_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if phone_number_create_request is not None:
            _body_params = phone_number_create_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'oAuth2'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/sites/{siteId}/phone-numbers',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def delete_site_phone_number_by_id(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        phone_number_id: Annotated[StrictInt, Field(description="The unique identifier of the phone number record.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Unassign/Delete Phone Number from Site

        Removes a phone number assignment from the site. The number may return to inventory.

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param phone_number_id: The unique identifier of the phone number record. (required)
        :type phone_number_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_site_phone_number_by_id_serialize(
            site_id=site_id,
            phone_number_id=phone_number_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '401': "Error",
            '403': "Error",
            '404': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def delete_site_phone_number_by_id_with_http_info(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        phone_number_id: Annotated[StrictInt, Field(description="The unique identifier of the phone number record.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Unassign/Delete Phone Number from Site

        Removes a phone number assignment from the site. The number may return to inventory.

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param phone_number_id: The unique identifier of the phone number record. (required)
        :type phone_number_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_site_phone_number_by_id_serialize(
            site_id=site_id,
            phone_number_id=phone_number_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '401': "Error",
            '403': "Error",
            '404': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def delete_site_phone_number_by_id_without_preload_content(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        phone_number_id: Annotated[StrictInt, Field(description="The unique identifier of the phone number record.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Unassign/Delete Phone Number from Site

        Removes a phone number assignment from the site. The number may return to inventory.

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param phone_number_id: The unique identifier of the phone number record. (required)
        :type phone_number_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete_site_phone_number_by_id_serialize(
            site_id=site_id,
            phone_number_id=phone_number_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '204': None,
            '401': "Error",
            '403': "Error",
            '404': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _delete_site_phone_number_by_id_serialize(
        self,
        site_id,
        phone_number_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if site_id is not None:
            _path_params['siteId'] = site_id
        if phone_number_id is not None:
            _path_params['phoneNumberId'] = phone_number_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'oAuth2'
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/sites/{siteId}/phone-numbers/{phoneNumberId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def get_site_phone_number_by_id(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        phone_number_id: Annotated[StrictInt, Field(description="The unique identifier of the phone number record.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SchemasPhoneNumberResponse:
        """Get Phone Number Details

        Retrieves detailed information for a specific phone number assigned to a site.

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param phone_number_id: The unique identifier of the phone number record. (required)
        :type phone_number_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_site_phone_number_by_id_serialize(
            site_id=site_id,
            phone_number_id=phone_number_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SchemasPhoneNumberResponse",
            '401': "Error",
            '403': "Error",
            '404': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_site_phone_number_by_id_with_http_info(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        phone_number_id: Annotated[StrictInt, Field(description="The unique identifier of the phone number record.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SchemasPhoneNumberResponse]:
        """Get Phone Number Details

        Retrieves detailed information for a specific phone number assigned to a site.

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param phone_number_id: The unique identifier of the phone number record. (required)
        :type phone_number_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_site_phone_number_by_id_serialize(
            site_id=site_id,
            phone_number_id=phone_number_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SchemasPhoneNumberResponse",
            '401': "Error",
            '403': "Error",
            '404': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_site_phone_number_by_id_without_preload_content(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        phone_number_id: Annotated[StrictInt, Field(description="The unique identifier of the phone number record.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Phone Number Details

        Retrieves detailed information for a specific phone number assigned to a site.

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param phone_number_id: The unique identifier of the phone number record. (required)
        :type phone_number_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_site_phone_number_by_id_serialize(
            site_id=site_id,
            phone_number_id=phone_number_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SchemasPhoneNumberResponse",
            '401': "Error",
            '403': "Error",
            '404': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _get_site_phone_number_by_id_serialize(
        self,
        site_id,
        phone_number_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if site_id is not None:
            _path_params['siteId'] = site_id
        if phone_number_id is not None:
            _path_params['phoneNumberId'] = phone_number_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'oAuth2'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/sites/{siteId}/phone-numbers/{phoneNumberId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def list_site_available_phone_numbers(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        user_id: Annotated[Optional[StrictInt], Field(description="Optional. ID of the user to filter available phone numbers by.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> AvailablePhoneNumberListResponse:
        """List Available Phone Numbers for a Site

        Retrieves phone numbers that are available for assignment to the specified site.  Can be optionally filtered by user ID.

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param user_id: Optional. ID of the user to filter available phone numbers by.
        :type user_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_site_available_phone_numbers_serialize(
            site_id=site_id,
            user_id=user_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AvailablePhoneNumberListResponse",
            '401': "Error",
            '403': "Error",
            '404': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def list_site_available_phone_numbers_with_http_info(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        user_id: Annotated[Optional[StrictInt], Field(description="Optional. ID of the user to filter available phone numbers by.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[AvailablePhoneNumberListResponse]:
        """List Available Phone Numbers for a Site

        Retrieves phone numbers that are available for assignment to the specified site.  Can be optionally filtered by user ID.

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param user_id: Optional. ID of the user to filter available phone numbers by.
        :type user_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_site_available_phone_numbers_serialize(
            site_id=site_id,
            user_id=user_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AvailablePhoneNumberListResponse",
            '401': "Error",
            '403': "Error",
            '404': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def list_site_available_phone_numbers_without_preload_content(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        user_id: Annotated[Optional[StrictInt], Field(description="Optional. ID of the user to filter available phone numbers by.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List Available Phone Numbers for a Site

        Retrieves phone numbers that are available for assignment to the specified site.  Can be optionally filtered by user ID.

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param user_id: Optional. ID of the user to filter available phone numbers by.
        :type user_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_site_available_phone_numbers_serialize(
            site_id=site_id,
            user_id=user_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "AvailablePhoneNumberListResponse",
            '401': "Error",
            '403': "Error",
            '404': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _list_site_available_phone_numbers_serialize(
        self,
        site_id,
        user_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if site_id is not None:
            _path_params['siteId'] = site_id
        # process the query parameters
        if user_id is not None:
            
            _query_params.append(('userId', user_id))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'oAuth2'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/sites/{siteId}/phone-numbers/available',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def list_site_phone_numbers(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        page: Annotated[Optional[StrictInt], Field(description="Page number for pagination.")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="Number of phone numbers per page.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PaginatedPhoneNumberListResponse:
        """List Phone Numbers for a Site

        Retrieve a paginated list of all phone numbers assigned to the specified site.

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param page: Page number for pagination.
        :type page: int
        :param page_size: Number of phone numbers per page.
        :type page_size: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_site_phone_numbers_serialize(
            site_id=site_id,
            page=page,
            page_size=page_size,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PaginatedPhoneNumberListResponse",
            '401': "Error",
            '403': "Error",
            '404': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def list_site_phone_numbers_with_http_info(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        page: Annotated[Optional[StrictInt], Field(description="Page number for pagination.")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="Number of phone numbers per page.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PaginatedPhoneNumberListResponse]:
        """List Phone Numbers for a Site

        Retrieve a paginated list of all phone numbers assigned to the specified site.

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param page: Page number for pagination.
        :type page: int
        :param page_size: Number of phone numbers per page.
        :type page_size: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_site_phone_numbers_serialize(
            site_id=site_id,
            page=page,
            page_size=page_size,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PaginatedPhoneNumberListResponse",
            '401': "Error",
            '403': "Error",
            '404': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def list_site_phone_numbers_without_preload_content(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        page: Annotated[Optional[StrictInt], Field(description="Page number for pagination.")] = None,
        page_size: Annotated[Optional[StrictInt], Field(description="Number of phone numbers per page.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List Phone Numbers for a Site

        Retrieve a paginated list of all phone numbers assigned to the specified site.

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param page: Page number for pagination.
        :type page: int
        :param page_size: Number of phone numbers per page.
        :type page_size: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._list_site_phone_numbers_serialize(
            site_id=site_id,
            page=page,
            page_size=page_size,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PaginatedPhoneNumberListResponse",
            '401': "Error",
            '403': "Error",
            '404': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _list_site_phone_numbers_serialize(
        self,
        site_id,
        page,
        page_size,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if site_id is not None:
            _path_params['siteId'] = site_id
        # process the query parameters
        if page is not None:
            
            _query_params.append(('page', page))
            
        if page_size is not None:
            
            _query_params.append(('page_size', page_size))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
            'oAuth2'
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/sites/{siteId}/phone-numbers',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def transfer_site_phone_number(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        phone_number_id: Annotated[StrictInt, Field(description="The unique identifier of the phone number record.")],
        new_site_id: Annotated[StrictInt, Field(description="The unique identifier of the target site for the phone number.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SchemasPhoneNumberResponse:
        """Transfer Phone Number to Another Site

        Moves an assigned phone number from its current site (specified in the path)  to a new target site (specified in the request body) within the same customer account. 

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param phone_number_id: The unique identifier of the phone number record. (required)
        :type phone_number_id: int
        :param new_site_id: The unique identifier of the target site for the phone number. (required)
        :type new_site_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._transfer_site_phone_number_serialize(
            site_id=site_id,
            phone_number_id=phone_number_id,
            new_site_id=new_site_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SchemasPhoneNumberResponse",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '404': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def transfer_site_phone_number_with_http_info(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        phone_number_id: Annotated[StrictInt, Field(description="The unique identifier of the phone number record.")],
        new_site_id: Annotated[StrictInt, Field(description="The unique identifier of the target site for the phone number.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SchemasPhoneNumberResponse]:
        """Transfer Phone Number to Another Site

        Moves an assigned phone number from its current site (specified in the path)  to a new target site (specified in the request body) within the same customer account. 

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param phone_number_id: The unique identifier of the phone number record. (required)
        :type phone_number_id: int
        :param new_site_id: The unique identifier of the target site for the phone number. (required)
        :type new_site_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._transfer_site_phone_number_serialize(
            site_id=site_id,
            phone_number_id=phone_number_id,
            new_site_id=new_site_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SchemasPhoneNumberResponse",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '404': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def transfer_site_phone_number_without_preload_content(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        phone_number_id: Annotated[StrictInt, Field(description="The unique identifier of the phone number record.")],
        new_site_id: Annotated[StrictInt, Field(description="The unique identifier of the target site for the phone number.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Transfer Phone Number to Another Site

        Moves an assigned phone number from its current site (specified in the path)  to a new target site (specified in the request body) within the same customer account. 

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param phone_number_id: The unique identifier of the phone number record. (required)
        :type phone_number_id: int
        :param new_site_id: The unique identifier of the target site for the phone number. (required)
        :type new_site_id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._transfer_site_phone_number_serialize(
            site_id=site_id,
            phone_number_id=phone_number_id,
            new_site_id=new_site_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SchemasPhoneNumberResponse",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '404': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _transfer_site_phone_number_serialize(
        self,
        site_id,
        phone_number_id,
        new_site_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if site_id is not None:
            _path_params['siteId'] = site_id
        if phone_number_id is not None:
            _path_params['phoneNumberId'] = phone_number_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        if new_site_id is not None:
            _form_params.append(('new_site_id', new_site_id))
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/x-www-form-urlencoded'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'oAuth2'
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/sites/{siteId}/phone-numbers/{phoneNumberId}/transfer',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    def update_site_phone_number_by_id(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        phone_number_id: Annotated[StrictInt, Field(description="The unique identifier of the phone number record.")],
        phone_number_update_request: Annotated[PhoneNumberUpdateRequest, Field(description="Phone number attributes to update.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SchemasPhoneNumberResponse:
        """Update Phone Number Configuration

        Updates configuration for a specific phone number. Only provided fields are changed.

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param phone_number_id: The unique identifier of the phone number record. (required)
        :type phone_number_id: int
        :param phone_number_update_request: Phone number attributes to update. (required)
        :type phone_number_update_request: PhoneNumberUpdateRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_site_phone_number_by_id_serialize(
            site_id=site_id,
            phone_number_id=phone_number_id,
            phone_number_update_request=phone_number_update_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SchemasPhoneNumberResponse",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '404': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def update_site_phone_number_by_id_with_http_info(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        phone_number_id: Annotated[StrictInt, Field(description="The unique identifier of the phone number record.")],
        phone_number_update_request: Annotated[PhoneNumberUpdateRequest, Field(description="Phone number attributes to update.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SchemasPhoneNumberResponse]:
        """Update Phone Number Configuration

        Updates configuration for a specific phone number. Only provided fields are changed.

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param phone_number_id: The unique identifier of the phone number record. (required)
        :type phone_number_id: int
        :param phone_number_update_request: Phone number attributes to update. (required)
        :type phone_number_update_request: PhoneNumberUpdateRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_site_phone_number_by_id_serialize(
            site_id=site_id,
            phone_number_id=phone_number_id,
            phone_number_update_request=phone_number_update_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SchemasPhoneNumberResponse",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '404': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def update_site_phone_number_by_id_without_preload_content(
        self,
        site_id: Annotated[StrictStr, Field(description="The unique identifier of the site.")],
        phone_number_id: Annotated[StrictInt, Field(description="The unique identifier of the phone number record.")],
        phone_number_update_request: Annotated[PhoneNumberUpdateRequest, Field(description="Phone number attributes to update.")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Update Phone Number Configuration

        Updates configuration for a specific phone number. Only provided fields are changed.

        :param site_id: The unique identifier of the site. (required)
        :type site_id: str
        :param phone_number_id: The unique identifier of the phone number record. (required)
        :type phone_number_id: int
        :param phone_number_update_request: Phone number attributes to update. (required)
        :type phone_number_update_request: PhoneNumberUpdateRequest
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_site_phone_number_by_id_serialize(
            site_id=site_id,
            phone_number_id=phone_number_id,
            phone_number_update_request=phone_number_update_request,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SchemasPhoneNumberResponse",
            '400': "Error",
            '401': "Error",
            '403': "Error",
            '404': "Error",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _update_site_phone_number_by_id_serialize(
        self,
        site_id,
        phone_number_id,
        phone_number_update_request,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if site_id is not None:
            _path_params['siteId'] = site_id
        if phone_number_id is not None:
            _path_params['phoneNumberId'] = phone_number_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if phone_number_update_request is not None:
            _body_params = phone_number_update_request


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
            'oAuth2'
        ]

        return self.api_client.param_serialize(
            method='PUT',
            resource_path='/sites/{siteId}/phone-numbers/{phoneNumberId}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

"""
    Ringer Business API

    This API provides comprehensive endpoints for the Ringer Business system. It includes functionality for managing customer accounts, addresses, sites,  phone numbers, payments, users, and other business resources.  Operations typically require authentication and appropriate permissions. 

    The version of the OpenAPI document: 1.0.4
    Contact: support@ringer.tel
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from ringer_business.api.packages_api import PackagesApi


class TestPackagesApi(unittest.TestCase):
    """PackagesApi unit test stubs"""

    def setUp(self) -> None:
        self.api = PackagesApi()

    def tearDown(self) -> None:
        pass

    def test_get_package_by_id(self) -> None:
        """Test case for get_package_by_id

        Get Package Details
        """
        pass

    def test_list_packages(self) -> None:
        """Test case for list_packages

        List Packages
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Ringer Business API - Plans Management

    This API provides endpoints for managing subscription plans associated with a customer account. It allows developers to: - List currently active plan(s) for an account. - List all plans available to an account. - Schedule a plan change for the next billing cycle. - Remove the current plan from an account. - Retrieve details of a specific plan. - Manage packages (add-ons) associated with a plan.  Plan management is crucial for defining the services and features available to a customer. 

    The version of the OpenAPI document: 1.0.0
    Contact: support@ringer.tel
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from ringer_business.models.item_response import ItemResponse
from ringer_business.models.link import Link
from ringer_business.models.plan_response_all_of_packages import PlanResponseAllOfPackages
from typing import Optional, Set
from typing_extensions import Self

class PlanResponse(BaseModel):
    """
    Detailed information about a subscription plan.
    """ # noqa: E501
    id: StrictInt
    name: StrictStr
    description: Optional[StrictStr] = None
    total_cost: StrictStr
    currency: StrictStr
    removable: StrictBool
    active: StrictBool
    approved: StrictBool
    finalized: StrictBool
    items: Optional[List[ItemResponse]] = Field(default=None, description="A list of individual billable items or features included or defined by this plan.")
    packages: Optional[PlanResponseAllOfPackages] = None
    links: Optional[List[Link]] = Field(default=None, description="HATEOAS links related to this plan.")
    __properties: ClassVar[List[str]] = ["id", "name", "description", "total_cost", "currency", "removable", "active", "approved", "finalized", "items", "packages", "links"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PlanResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in items (list)
        _items = []
        if self.items:
            for _item_items in self.items:
                if _item_items:
                    _items.append(_item_items.to_dict())
            _dict['items'] = _items
        # override the default output from pydantic by calling `to_dict()` of packages
        if self.packages:
            _dict['packages'] = self.packages.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in links (list)
        _items = []
        if self.links:
            for _item_links in self.links:
                if _item_links:
                    _items.append(_item_links.to_dict())
            _dict['links'] = _items
        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict['description'] = None

        # set to None if items (nullable) is None
        # and model_fields_set contains the field
        if self.items is None and "items" in self.model_fields_set:
            _dict['items'] = None

        # set to None if packages (nullable) is None
        # and model_fields_set contains the field
        if self.packages is None and "packages" in self.model_fields_set:
            _dict['packages'] = None

        # set to None if links (nullable) is None
        # and model_fields_set contains the field
        if self.links is None and "links" in self.model_fields_set:
            _dict['links'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PlanResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "name": obj.get("name"),
            "description": obj.get("description"),
            "total_cost": obj.get("total_cost"),
            "currency": obj.get("currency") if obj.get("currency") is not None else 'USD',
            "removable": obj.get("removable"),
            "active": obj.get("active"),
            "approved": obj.get("approved"),
            "finalized": obj.get("finalized"),
            "items": [ItemResponse.from_dict(_item) for _item in obj["items"]] if obj.get("items") is not None else None,
            "packages": PlanResponseAllOfPackages.from_dict(obj["packages"]) if obj.get("packages") is not None else None,
            "links": [Link.from_dict(_item) for _item in obj["links"]] if obj.get("links") is not None else None
        })
        return _obj





# coding: utf-8

"""
    Ringer Business API

    This API provides comprehensive endpoints for the Ringer Business system. It includes functionality for managing customer accounts, addresses, sites,  phone numbers, payments, users, and other business resources.  Operations typically require authentication and appropriate permissions. 

    The version of the OpenAPI document: 1.0.4
    Contact: support@ringer.tel
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from ringer_business.api.site_features_schedules_api import SiteFeaturesSchedulesApi


class TestSiteFeaturesSchedulesApi(unittest.TestCase):
    """SiteFeaturesSchedulesApi unit test stubs"""

    def setUp(self) -> None:
        self.api = SiteFeaturesSchedulesApi()

    def tearDown(self) -> None:
        pass

    def test_create_site_schedule(self) -> None:
        """Test case for create_site_schedule

        Create Schedule for a Site
        """
        pass

    def test_delete_site_schedule_by_id(self) -> None:
        """Test case for delete_site_schedule_by_id

        Delete a Schedule
        """
        pass

    def test_get_site_schedule_by_id(self) -> None:
        """Test case for get_site_schedule_by_id

        Get Schedule Details
        """
        pass

    def test_list_site_available_schedules(self) -> None:
        """Test case for list_site_available_schedules

        List Available Schedules for a Site
        """
        pass

    def test_list_site_schedules(self) -> None:
        """Test case for list_site_schedules

        List Schedules for a Site
        """
        pass

    def test_transfer_site_schedule(self) -> None:
        """Test case for transfer_site_schedule

        Transfer Schedule to Another Site
        """
        pass

    def test_update_site_schedule_by_id(self) -> None:
        """Test case for update_site_schedule_by_id

        Update Schedule
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Ringer Business API

    This API provides comprehensive endpoints for the Ringer Business system. It includes functionality for managing customer accounts, addresses, sites,  phone numbers, payments, users, and other business resources.  Operations typically require authentication and appropriate permissions. 

    The version of the OpenAPI document: 1.0.4
    Contact: support@ringer.tel
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

class SeatCreateRequest(BaseModel):
    """
    Data required to create a new seat for a user on a site using a template.
    """ # noqa: E501
    name: StrictStr = Field(description="A descriptive name for the seat assignment.")
    template_id: StrictInt = Field(description="Identifier of the service template to use for provisioning this seat.")
    user_id: Optional[StrictInt] = Field(default=None, description="Identifier of the user to whom this seat is assigned.")
    __properties: ClassVar[List[str]] = ["name", "template_id", "user_id"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SeatCreateRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if user_id (nullable) is None
        # and model_fields_set contains the field
        if self.user_id is None and "user_id" in self.model_fields_set:
            _dict['user_id'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SeatCreateRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "name": obj.get("name"),
            "template_id": obj.get("template_id"),
            "user_id": obj.get("user_id")
        })
        return _obj





# coding: utf-8

"""
    Ringer Business API

    This API provides comprehensive endpoints for the Ringer Business system. It includes functionality for managing customer accounts, addresses, sites,  phone numbers, payments, users, and other business resources.  Operations typically require authentication and appropriate permissions. 

    The version of the OpenAPI document: 1.0.4
    Contact: support@ringer.tel
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from ringer_business.models.country_codes_components_schemas_error import CountryCodesComponentsSchemasError

class TestCountryCodesComponentsSchemasError(unittest.TestCase):
    """CountryCodesComponentsSchemasError unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> CountryCodesComponentsSchemasError:
        """Test CountryCodesComponentsSchemasError
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `CountryCodesComponentsSchemasError`
        """
        model = CountryCodesComponentsSchemasError()
        if include_optional:
            return CountryCodesComponentsSchemasError(
                status = 'error',
                error = 'Failed to retrieve country codes.',
                error_code = 'COUNTRY_CODE_LIST_ERROR'
            )
        else:
            return CountryCodesComponentsSchemasError(
                status = 'error',
                error = 'Failed to retrieve country codes.',
                error_code = 'COUNTRY_CODE_LIST_ERROR',
        )
        """

    def testCountryCodesComponentsSchemasError(self):
        """Test CountryCodesComponentsSchemasError"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Ringer Business API

    This API provides comprehensive endpoints for the Ringer Business system. It includes functionality for managing customer accounts, addresses, sites,  phone numbers, payments, users, and other business resources.  Operations typically require authentication and appropriate permissions. 

    The version of the OpenAPI document: 1.0.4
    Contact: support@ringer.tel
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from ringer_business.api.accounts_api import AccountsApi


class TestAccountsApi(unittest.TestCase):
    """AccountsApi unit test stubs"""

    def setUp(self) -> None:
        self.api = AccountsApi()

    def tearDown(self) -> None:
        pass

    def test_activate_account(self) -> None:
        """Test case for activate_account

        Activate Current Account
        """
        pass

    def test_close_account(self) -> None:
        """Test case for close_account

        Close Current Account
        """
        pass

    def test_get_account_details(self) -> None:
        """Test case for get_account_details

        Get Current Account Details
        """
        pass

    def test_update_account(self) -> None:
        """Test case for update_account

        Update Current Account Information
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Ringer Business API

    This API provides comprehensive endpoints for the Ringer Business system. It includes functionality for managing customer accounts, addresses, sites,  phone numbers, payments, users, and other business resources.  Operations typically require authentication and appropriate permissions. 

    The version of the OpenAPI document: 1.0.4
    Contact: support@ringer.tel
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from ringer_business.api.country_codes_api import CountryCodesApi


class TestCountryCodesApi(unittest.TestCase):
    """CountryCodesApi unit test stubs"""

    def setUp(self) -> None:
        self.api = CountryCodesApi()

    def tearDown(self) -> None:
        pass

    def test_list_available_country_codes(self) -> None:
        """Test case for list_available_country_codes

        List Available Country Codes
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Ringer Business API - Authentication

    The Ringer Business API uses OAuth 2.0 to authenticate and authorize client applications. This section details the endpoints for obtaining and revoking access tokens.  **Supported Grant Types for Obtaining Access Tokens (`/auth/token`):**  *   **Password Grant:**     For trusted first-party applications where users directly provide their credentials.     Requires: `grant_type=\"password\"`, `username`, `password`, `client_id`, `client_secret`.  *   **Client Credentials Grant:**     For server-to-server authentication where the client is acting on its own behalf.     Requires: `grant_type=\"client_credentials\"`, `client_id`, `client_secret`.  *   **Refresh Token Grant:**     To obtain a new access token using a previously issued refresh token.     Requires: `grant_type=\"refresh_token\"`, `refresh_token`, `client_id`, `client_secret`.  Access tokens are typically short-lived (e.g., 2 hours). Use the refresh token to maintain an active session without requiring users to re-authenticate frequently.  **Important Security Note:** Your `client_id` and `client_secret` are confidential. Secure them appropriately and never expose them in client-side code or public repositories. 

    The version of the OpenAPI document: 1.0.0
    Contact: support@ringer.tel
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, SecretStr, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

class TokenRequest(BaseModel):
    """
    Parameters for requesting an OAuth 2.0 access token. Required fields vary by grant_type.
    """ # noqa: E501
    grant_type: StrictStr = Field(description="The OAuth 2.0 grant type.")
    username: Optional[StrictStr] = Field(default=None, description="User's email address (required for `password` grant).")
    password: Optional[SecretStr] = Field(default=None, description="User's password (required for `password` grant).")
    refresh_token: Optional[StrictStr] = Field(default=None, description="Refresh token (required for `refresh_token` grant).")
    client_id: StrictStr = Field(description="The application's client ID.")
    client_secret: StrictStr = Field(description="The application's client secret.")
    scope: Optional[StrictStr] = Field(default=None, description="(Optional) A space-separated list of requested scopes. The granted scopes may differ.")
    __properties: ClassVar[List[str]] = ["grant_type", "username", "password", "refresh_token", "client_id", "client_secret", "scope"]

    @field_validator('grant_type')
    def grant_type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in set(['password', 'client_credentials', 'refresh_token']):
            raise ValueError("must be one of enum values ('password', 'client_credentials', 'refresh_token')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of TokenRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if username (nullable) is None
        # and model_fields_set contains the field
        if self.username is None and "username" in self.model_fields_set:
            _dict['username'] = None

        # set to None if password (nullable) is None
        # and model_fields_set contains the field
        if self.password is None and "password" in self.model_fields_set:
            _dict['password'] = None

        # set to None if refresh_token (nullable) is None
        # and model_fields_set contains the field
        if self.refresh_token is None and "refresh_token" in self.model_fields_set:
            _dict['refresh_token'] = None

        # set to None if scope (nullable) is None
        # and model_fields_set contains the field
        if self.scope is None and "scope" in self.model_fields_set:
            _dict['scope'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TokenRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "grant_type": obj.get("grant_type"),
            "username": obj.get("username"),
            "password": obj.get("password"),
            "refresh_token": obj.get("refresh_token"),
            "client_id": obj.get("client_id"),
            "client_secret": obj.get("client_secret"),
            "scope": obj.get("scope")
        })
        return _obj





# coding: utf-8

"""
    Ringer Business API - Items Management

    This API provides endpoints for managing individual billable items or features. Items represent the most granular level of service components (e.g., a specific calling feature, a per-minute rate for a call type, a monthly fee for a service). These items can be part of packages or plans.  Currently, this API focuses on allowing updates to items, typically related to their pricing within a specific price deck context. 

    The version of the OpenAPI document: 1.0.0
    Contact: support@ringer.tel
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

class ItemCore(BaseModel):
    """
    ItemCore
    """ # noqa: E501
    id: StrictInt = Field(description="Unique identifier for the item.")
    name: StrictStr = Field(description="The human-readable name of the item or feature.")
    item_type: StrictStr = Field(description="The general category or type of the item (e.g., cdr, service_fee, feature_license).")
    item_rules: Optional[StrictStr] = Field(default=None, description="Specific rules or sub-classification for the item type that might affect rating or behavior.")
    item_price: StrictStr = Field(description="The price of this individual item (e.g., per unit, per month).")
    currency: StrictStr = Field(description="Three-letter ISO currency code for the item price.")
    price_deck_id: Optional[StrictInt] = Field(default=None, description="Identifier of the price deck this item pricing might belong to.")
    tax_category: Optional[StrictInt] = Field(default=None, description="Identifier for the tax category applicable to this item.")
    tax_code: Optional[StrictInt] = Field(default=None, description="Specific tax code applicable to this item.")
    __properties: ClassVar[List[str]] = ["id", "name", "item_type", "item_rules", "item_price", "currency", "price_deck_id", "tax_category", "tax_code"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ItemCore from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if item_rules (nullable) is None
        # and model_fields_set contains the field
        if self.item_rules is None and "item_rules" in self.model_fields_set:
            _dict['item_rules'] = None

        # set to None if price_deck_id (nullable) is None
        # and model_fields_set contains the field
        if self.price_deck_id is None and "price_deck_id" in self.model_fields_set:
            _dict['price_deck_id'] = None

        # set to None if tax_category (nullable) is None
        # and model_fields_set contains the field
        if self.tax_category is None and "tax_category" in self.model_fields_set:
            _dict['tax_category'] = None

        # set to None if tax_code (nullable) is None
        # and model_fields_set contains the field
        if self.tax_code is None and "tax_code" in self.model_fields_set:
            _dict['tax_code'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ItemCore from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "name": obj.get("name"),
            "item_type": obj.get("item_type"),
            "item_rules": obj.get("item_rules"),
            "item_price": obj.get("item_price"),
            "currency": obj.get("currency") if obj.get("currency") is not None else 'USD',
            "price_deck_id": obj.get("price_deck_id"),
            "tax_category": obj.get("tax_category"),
            "tax_code": obj.get("tax_code")
        })
        return _obj





# coding: utf-8

"""
    Ringer Business API

    This API provides comprehensive endpoints for the Ringer Business system. It includes functionality for managing customer accounts, addresses, sites,  phone numbers, payments, users, and other business resources.  Operations typically require authentication and appropriate permissions. 

    The version of the OpenAPI document: 1.0.4
    Contact: support@ringer.tel
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from ringer_business.api.site_features_menus_api import SiteFeaturesMenusApi


class TestSiteFeaturesMenusApi(unittest.TestCase):
    """SiteFeaturesMenusApi unit test stubs"""

    def setUp(self) -> None:
        self.api = SiteFeaturesMenusApi()

    def tearDown(self) -> None:
        pass

    def test_create_site_menu(self) -> None:
        """Test case for create_site_menu

        Create Menu for a Site
        """
        pass

    def test_delete_site_menu_by_id(self) -> None:
        """Test case for delete_site_menu_by_id

        Delete Menu
        """
        pass

    def test_get_site_menu_by_id(self) -> None:
        """Test case for get_site_menu_by_id

        Get Menu Details
        """
        pass

    def test_list_site_menus(self) -> None:
        """Test case for list_site_menus

        List Menus for a Site
        """
        pass

    def test_transfer_site_menu(self) -> None:
        """Test case for transfer_site_menu

        Transfer Menu to Another Site
        """
        pass

    def test_update_site_menu_by_id(self) -> None:
        """Test case for update_site_menu_by_id

        Update Menu Configuration
        """
        pass

    def test_upload_site_menu_greeting(self) -> None:
        """Test case for upload_site_menu_greeting

        Upload Menu Greeting
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Ringer Business API

    This API provides comprehensive endpoints for the Ringer Business system. It includes functionality for managing customer accounts, addresses, sites,  phone numbers, payments, users, and other business resources.  Operations typically require authentication and appropriate permissions. 

    The version of the OpenAPI document: 1.0.4
    Contact: support@ringer.tel
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from ringer_business.models.paginated_menu_list_response import PaginatedMenuListResponse

class TestPaginatedMenuListResponse(unittest.TestCase):
    """PaginatedMenuListResponse unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> PaginatedMenuListResponse:
        """Test PaginatedMenuListResponse
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `PaginatedMenuListResponse`
        """
        model = PaginatedMenuListResponse()
        if include_optional:
            return PaginatedMenuListResponse(
                menus = [
                    {"id":55,"site_id":6,"name":"Main Business Hours Menu","pin":"1234","fail_over_destination_code":"voicemailbox:general_vm","options":{"0":"ring_group:sales_rg","1":"extension:1001","*":"voicemailbox:operator_vm"},"greeting_url":"https://cdn.example.com/greetings/menu_55.mp3","links":[{"href":"/sites/6/menus/55","rel":"self","method":"GET"}]}
                    ],
                total_items = 20,
                total_pages = 2,
                current_page = 1,
                per_page = 10,
                links = [
                    ringer_business.models.link.Link(
                        href = '/example/resource/123', 
                        rel = 'self', 
                        method = 'GET', 
                        title = 'View Example Resource', )
                    ]
            )
        else:
            return PaginatedMenuListResponse(
                menus = [
                    {"id":55,"site_id":6,"name":"Main Business Hours Menu","pin":"1234","fail_over_destination_code":"voicemailbox:general_vm","options":{"0":"ring_group:sales_rg","1":"extension:1001","*":"voicemailbox:operator_vm"},"greeting_url":"https://cdn.example.com/greetings/menu_55.mp3","links":[{"href":"/sites/6/menus/55","rel":"self","method":"GET"}]}
                    ],
                total_items = 20,
                total_pages = 2,
                current_page = 1,
                per_page = 10,
        )
        """

    def testPaginatedMenuListResponse(self):
        """Test PaginatedMenuListResponse"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
